; ===============================================================
; Graphical Routines
; ===============================================================

; ---------------------------------------------------------------
; ClearBuff
; Clears buffers
; ---------------------------------------------------------------

ClearBuff:
.relocate SwapZone
	ld HL, PlotSScreen
	ld DE, PlotSScreen + 1
	ld BC, 768
	ld (HL), 0
	ldir
	
	ld HL, SaveSScreen
	ld DE, SaveSSCreen + 1
	ld BC, 768
	ld (HL), 0
	ldir
	ret
.endrelocate

; ---------------------------------------------------------------
; InvRectBlack
; Inverts rectangle area on both buffer
; Same I/O as InvRect
; ---------------------------------------------------------------

InvRectBlack:
.relocate SwapZone
;	push HL \ push BC
;	RCall( cInvRect )
;	pop BC \ pop HL
;	RCall( cInvRectDark )
;	ret
.endrelocate

; ---------------------------------------------------------------
; InvRectDark
; Inverts rectangle area on buffer 2
; Same I/O as InvRect
; ---------------------------------------------------------------

InvRectDark:
.relocate SwapZone
	call GrayMode
	call $ + 3 + 3 + 1 		; RCall( cInvRect )
	call NormMode
	ret
.endrelocate

; ---------------------------------------------------------------
; InvRect:
; Inverts a rectangle area using line routines
; Input: H = x coordinate
;        L = y coordinate
;        B = width
;        C = height
; ---------------------------------------------------------------

InvRect:
.relocate SwapZone
	set LineInvert, (IY + Asm_Flag2)
	call DrawRect
	res LineInvert, (IY + Asm_Flag2)
	ret
.endrelocate

; ---------------------------------------------------------------
; DrawBox:
; Draws a box
; Input: H = x coordinates
;	  L = y coordinates
;	  B = width
;	  C = height
; ---------------------------------------------------------------

DrawBox:
.relocate SwapZone
DrawBoxCall:
	call $ + DrawBoxEnd - DrawBoxCall	; DrawBoxOut
	
	inc H					;inner box
	inc L
	dec B
	dec B
	dec C
	dec C
		
	call HorizLine
	
	push HL
	ld A, L
	add A, C
	ld L, A
	call HorizLine
	pop HL
	
	push BC
	ld B, C
	call VertLine
	pop BC
	
	ld A, B
	add A, H
	dec A
	ld H, A
	ld B, C
	call VertLine
	ret
DrawBoxEnd:
.endrelocate
	
DrawBoxOut:					; subroutine - draws outer, black box
.relocate SwapZone
	RCall( cDrawBoxOut2 )			; DrawBoxOut2
.endrelocate
	
DrawBoxOut3:					; dark gray box
.relocate SwapZone
	call GrayMode
DrawBoxOut3Call:
	RCall( cDrawBoxOut2 )			 ; DrawBoxOut2
	call NormMode
	ret
DrawBoxOut3End:
.endrelocate

DrawBoxOut2:					; light gray box
.relocate SwapZone
	call HorizLine			;outer box
	
	push HL
	ld A, L
	add A, C
	ld L, A
	call HorizLine
	pop HL
	
	push BC
	ld B, C
	call VertLine
	pop BC
	
	push BC \ push HL
	ld A, B
	add A, H
	dec A
	ld H, A
	ld B, C
	call VertLine
	pop HL \ pop BC
	ret
.endrelocate
	
; ---------------------------------------------------------------
; CopyBuffSect:
; Copies part of Plot onto Save
; Input: A = x
;	  L = y
;	  B = width
; 	  C = height
; ---------------------------------------------------------------

CopyBuffSect:			; ~150 bytes
.relocate SwapZone
	push BC
	call GetPixel
	ld E, C		; E = x % 8
	pop BC
	
	bit Buffer2, (IY + Asm_Flag2)
	jr z, CopyBuffSectB1
CopyBuffSectB2:		; if Buffer 2
	push BC
	ld BC, PlotSScreen - SaveSScreen
	or A			; set c flag to 0
	sbc HL, BC		; HL = pt to pixel in SaveSScreen
	pop BC
	
CopyBuffSectB1:
	ld A, 8		; B = B - (8 - E)
	sub E			; B rounded down to nearest multiple of 8 (17 --> 16)
	ld D, A
	ld A, B
	sub D
	ld B, A
	
	push BC		; D = 1 >> (x % 8)
	ld B, E		; Ex: if x % 8 = 3,
	ld A, B		; then D = %00011111
	or A
	jr z, CopyBuffSectSkip01
	xor A
CopyBuffSectLp1:
	scf
	rra
	djnz CopyBuffSectLp1
	cpl
	
CopyBuffSectSkip01:
	pop BC
	ld D, A
	
	push HL
	push DE			; B = B / 8 - 1
	push BC			; A = inv(1 >> (B % 8))
	ld A, B			; Ex: if B % 8 = 3, 
	ld D, 8			; then A = %11100000
	RCall( cDiv )
	pop BC
	pop DE
	ld B, L
	push BC
	
	ld B, A
	or A
	jr z, CopyBuffSectSkip0
	xor A
CopyBuffSectLp:
	scf
	rra
	djnz CopyBuffSectLp
	
CopyBuffSectSkip0:
	pop BC
	pop HL
	
	inc B
	
CopyBuffSectLpY:
	push HL
	push BC
	push AF
	
	ld A, D			; For the first byte
	push DE
	call CopyBuffSectRout
	pop DE
	
	ld A, B			; If B == 0
	or A				; skip
	jr z, CopyBuffSectSkip00	; prevent djnz 256 times glitch
	
CopyBuffSectLpX:
	push DE
	ld A, (HL)
	push HL
	
	call CopyBuffSectSwitch
	ld (HL), A
	pop HL
	inc HL
	pop DE
	
	djnz CopyBuffSectLpX
	
CopyBuffSectSkip00:
	pop AF				; For the last byte
	push DE
	call CopyBuffSectRout
	pop DE
	
	pop BC
	pop HL
	dec C
	ret z
	push DE
	ld DE, 12
	add HL, DE
	pop DE
	jr CopyBuffSectLpY
	
CopyBuffSectRout:
	or A				; If A != 0
	ret z				; Then
	push HL			; Savebyte = (PlotByte && A) || (Savebyte && (not(A)))
	push BC
	push AF
	ld B, A
	ld A, (HL)
	and B
	
	call CopyBuffSectSwitch
	ld C, A
	ld A, B
	cpl	
	ld B, (HL)
	and B
	or C
	ld (HL), A
	pop AF
	pop BC
	pop HL
	dec B
	inc HL
	ret
	
CopyBuffSectSwitch:			; subroutine - switches to other buffer
	bit Buffer2, (IY + Asm_Flag2)
	jr nz, CopyBuffSectSwitch2
CopyBuffSectSwitch1:
	scf \ ccf	; HL = HL - (PlotSScreen - SaveSScreen)
	ld DE, 3156
	sbc HL, DE
	ret
CopyBuffSectSwitch2:
	ld DE, 3156
	add HL, DE	; HL = HL + (PlotSSCreen - SaveSScreen)
	ret
.endrelocate

; ---------------------------------------------------------------
; CopyBuffSect2
; Copies part of Plot onto Save
; Input: A = x
;	  L = y
;	  B = width
; 	  C = height
; ---------------------------------------------------------------

CopyBuffSect2:
.relocate SwapZone
	set Buffer2, (IY + Asm_Flag2)	; set to buffer 2
	RCall( cCopyBuffSect )
	res Buffer2, (IY + Asm_Flag2)	; reset buffer 2 flag
	ret
.endrelocate
	
; ---------------------------------------------------------------
; FillBuffSect
; Fills a section of both buffers
; Input: HL = location of top-left pt on PlotSScreen
;	  A = fill byte
;	  B = width / 8
; 	  C = height
; ---------------------------------------------------------------

FillBuffSect:
.relocate SwapZone
FillBuffSectY:
	push BC
	push HL
FillBuffSectX:
	push HL
	ld (HL), A
	scf \ ccf	; HL = HL - (PlotSScreen - SaveSScreen)
	ld DE, 3156
	sbc HL, DE
	ld (HL), A
	pop HL
	inc HL
	djnz FillBuffSectX
	
	pop HL
	pop BC
	dec C
	ret z
	ld DE, 12
	add HL, DE
	jr FillBuffSectY
.endrelocate

; ---------------------------------------------------------------
; FillBuffSectG
; Fills a section of a single gs buffer
; Input: HL = location of top-left pt on Save/PlotSScreen
;	  A = fill byte
;	  B = width / 8
; 	  C = height
; ---------------------------------------------------------------

FillBuffSectG:
.relocate SwapZone
FillBuffSectGY:
	push BC
	push HL
FillBuffSectGX:
	ld (HL), A
	inc HL
	djnz FillBuffSectGX
	
	pop HL
	pop BC
	dec C
	ret z
	ld DE, 12
	add HL, DE
	jr FillBuffSectGY
.endrelocate

;==========================================================
; gsLargeSprite
;==========================================================
;=== gsLargeSprite 1.0 by Durk Kingma
;thanks to Joe Wingbermuehle for his ionPutSprite routine
;
; puts large Grayscale sprite
; l=yc
; a=xc
; ix holds pointer
;-----> Draw a picture
;Input:
;	ix->sprite
;	a=x
;	l=y
;Output: nothing
; All registers are destroyed except bc', de', hl'

gsLargeSprite:
.relocate SwapZone
	ld B, (IX)
	inc IX
	ld C, (IX)
	inc IX
	
	push af
	push hl
	push bc
	
	call gsLargeSpriteCall		; call w/ PlotSScreen
	call GrayMode
	
	pop bc
	pop hl
	pop af
	
	call gsLargeSpriteCall		; call w/ SaveSScreen
	call NormMode
	
	ret

gsLargeSpriteCall:
	ex	af,af'
	ld	a,c
	push	af
	ex	af,af'
	ld	e,l
	ld	h,$00
	ld	d,h
	add	hl,de ;x12
	add	hl,de ;
	add	hl,hl ;
	add	hl,hl ;
	ld	e,a
	and	$07
	ld	c,a
	srl	e
	srl	e
	srl	e
	add	hl,de
	ld	de, (DrawBuff) ; SaveSScreen
	add	hl,de
gsLargeSpriteLoop1:
	push	hl
gsLargeSpriteLoop2:
	ld	d,(ix)
	ld	e,$00
	ld	a,c
	or	a
	jr	z,gsLargeSpriteSkip1
gsLargeSpriteLoop3:
	srl	d
	rr	e
	dec	a
	jr	nz,gsLargeSpriteLoop3
gsLargeSpriteSkip1:
	ld	a,(hl)
	xor	d
	ld	(hl),a
	inc	hl
	ld	a,(hl)
	xor	e
	ld	(hl),a
	inc	ix
	ex	af,af'
	dec	a
	push	af
	ex	af,af'
	pop	af
	jr	nz,gsLargeSpriteLoop2
	pop	hl
	pop	af
	push	af
	ex	af,af'
	ld	de,$0C
	add	hl,de
	djnz	gsLargeSpriteLoop1
	pop	af
	
	ld a,FREQ \ out ($04),a	; set timer freq to 11
	ld a,ACKN \ out ($03),a	; acknowledge interrupt
	ld a,MASK \ out ($03),a	; set interrupt mask
	ret
.endrelocate

;==========================================================
; gsPutSprite
;==========================================================
;=== gsPutSprite 1.0 by Durk Kingma
;thanks to Joe Wingbermuehle for his ionPutSprite routine
;
; puts XOR grayscale sprite
; l=yc
; a=xc
; ix holds pointer

gsPutSprite:
.relocate SwapZone
	ld B, (IX)
	inc IX

	push af
	push hl
	push bc

	call	ionPutSprite

	pop bc
	pop hl
	pop af

	ld	e,l
	ld	h,$00
	ld	d,h
	add	hl,de
	add	hl,de
	add	hl,hl
	add	hl,hl
	ld	e,a
	and	$07
	ld	c,a
	srl	e
	srl	e
	srl	e
	add	hl,de
	ld	de,SaveSScreen
	add	hl,de
gsPutSpriteLoop1:
pgxs_sl1:	ld	d,(ix)
	ld	e,$00
	ld	a,c
	or	a
	jr	z,gsPutSpriteSkip1
gsPutSpriteLoop2:
	srl	d
	rr	e
	dec	a
	jr	nz,gsPutSpriteLoop2
gsPutSpriteSkip1:
	ld	a,(hl)
	xor	d
	ld	(hl),a
	inc	hl
	ld	a,(hl)
	xor	e
	ld	(hl),a
	ld	de,$0B
	add	hl,de
	inc	ix
	djnz	gsPutSpriteLoop1

	ret
.endrelocate

; ---------------------------------------------------------------
; gsDisable
; Disables grayscale, restore contrast
; ---------------------------------------------------------------

gsDisable:
.relocate SwapZone
	di
	im 1
	ld A, 6 \ out ($04), A		; set timer freq to 11, default
	ei
	
	ld HL, Contrast
	ld A, (SaveContrast)				; A = saved contrast value
	ld (HL), A					; restore contrast, memory location
	add A, $D8					; A = A + $D8
	;call $0B					; call routine at address 0x0B
	out (16), A					; restore contrast, output port
	
	ret
.endrelocate