; ---------------------------------------------------------------
; RTextbox:
; Gets pt to text from code stream, executes RCall( cTextbox )
; Saves all registers
; ---------------------------------------------------------------

TextShadowStart:			; ~ 120 bytes total
.relocate TextShadow
RTextbox:				; ~ 32 bytes
	push AF
	push BC
	push DE
	push HL
	
	; stack structure:
	; HL param		<-- SP points here
	; DE param
	; BC param
	; AF param
	; return address
	
	ld HL, 8 + 1
	add HL, SP			; HL pts to upper byte ; TODO: combine w/ RCall routine?
	ld D, (HL)
	dec HL
	ld E, (HL)			; DE = return address
	inc DE				; DE +=2
	inc DE				; move DE past .dw lbl in code stream
	ld (HL), E
	inc HL
	ld (HL), D			; modify return address
	
	dec DE				; move DE to pt to txt
	ld A, (DE)			; HL = txt
	ld H, A
	dec DE
	ld A, (DE)
	ld L, A
	
	RCall( cTextbox )
	
	pop HL
	pop DE
	pop BC
	pop AF
	ret
	
; ---------------------------------------------------------------
; RTextboxRet:
; Gets pt to text from code stream, executes RCall( cTextbox )
; Destroys all registers
; ---------------------------------------------------------------

RTextboxRet:				; ~12 bytes
	; stack structure:
	; return address 1	<-- SP points here
	; return address 2
	
	ex (SP), HL			; HL = (SP)
	ld A, (HL)
	inc HL				; HL pts to txt MSB
	ld H, (HL)
	ld L, A			; HL = txt
	
	RCall( cTextbox )
	
	pop HL				; pop off return address 1
	ret				; ret to return address 2
	
; ---------------------------------------------------------------
; GetPixelGray:
; Gets pixel in buffer 2
; Same I/O as GetPixel
; ---------------------------------------------------------------

GetPixelGray:			; ~ 9 bytes
	call GrayMode
	call GetPixel
	jp NormMode
	
; ---------------------------------------------------------------
; GetPixel:
; Gets a pixel in PlotSScreen
; Input: A = x coordinate
;	  L = y coordinate
; Output: HL = location in buffer
;	   A = bitmask
;	   C = x mod 8
; Preserves BC and DE
; ---------------------------------------------------------------

GetPixel:			; ~36 bytes
	push DE
	ld H, 0
	push HL		;HL -> DE
	pop DE
	add HL, HL		;HL = y * 12
	add HL, DE
	add HL, HL
	add HL, HL
	
	ld E, A		;HL = HL + x * 8
	srl E
	srl E
	srl E
	add HL, DE
	
	ld DE, (DrawBuff)
	add HL, DE
	pop DE
	
	and 7
	ld C, A		;output x mod 8 to C
	ld A, $80
	ret z
	
	push BC
	ld B, C
GetPixelLp:
	rrca
	djnz GetPixelLp
	pop BC
	ret

; ---------------------------------------------------------------
; GrayMode:
; Changes graphics routines to gray mode (SaveSScreen)
; Note: Normal mode is always assumed
; ---------------------------------------------------------------

GrayMode:
	push HL		; ~9 bytes
	ld HL, SaveSScreen
	ld (DrawBuff), HL
	pop HL
	ret
	
; ---------------------------------------------------------------
; NormMode:
; Changes graphics routines to normal mode (PlotSScreen)
; Note: Normal mode is always assumed
; ---------------------------------------------------------------

NormMode:
	push HL		; ~9 bytes
	ld HL, PlotSScreen
	ld (DrawBuff), HL
	pop HL
	ret
	
; ---------------------------------------------------------------
; RParty:
; Swaps Overworld into executable RAM, brings up party menu
; ---------------------------------------------------------------

RParty:
	call Relocate		; ~8 bytes
	call Party
	jp Relocate
	;call Relocate
	;ret
	
; ===============================================================
; Math
; ===============================================================

; ---------------------------------------------------------------
; RCpHLDE
; Compare HL and DE
; ---------------------------------------------------------------

RCpHLDE:			; ~5 bytes
	push HL
	or A
	sbc HL, DE
	pop HL
	ret
.endrelocate

TextShadowEnd:

Routines_End:					; Must be at end of routines code

; ---------------------------------------------------------------
; Analysis
; ---------------------------------------------------------------

.echo "Frontier PKMN: "
.echo fPKMN5End - fPKMN1
.echo " bytes.\n"

.echo "Pokedex: "
.echo dPokedexEnd - dPokedex
.echo " bytes.\n"

.echo "Moves: "
.echo mdMovesEnd - mdMovesBeg
.echo " bytes.\n"

.echo "Text: "
.echo ttTextEnd - ttText
.echo " bytes.\n"

.echo "Routines: "
.echo Routines_End - Routines_Start
.echo " bytes.\n"

.echo "Total: "
.echo Routines_End - fPKMN1
.echo " bytes. ("
.echo 100 * (Routines_End - fPKMN1 + 19) / (24131)
.echo "%)\n"
