; ===============================================================
; Attack Effect
; ===============================================================

; ---------------------------------------------------------------
; AttackEffect
; Determines attack effects
; Input: (AtkMove) = attacking move, (StatVars), etc
; ---------------------------------------------------------------

AttackEffect:
	ld A, (AtkMove)
	cp 181				; if A >= 181 (mHitInConfusion, mFutureHit)
	ret nc				; then return immediately
	
	cp mHiJumpKick		; if move is high jump kick, then effect applies on protect
	jr z, AttackEffectSkipProtect
	
	call IsTargetProtect		; get target protect status, protect flag, preserves A
	ret c				; ret if protected
	
AttackEffectSkipProtect:
	RCall( cGetMoveEffect )	; A, B = Move effect ID, HL = pt to move effect ID
	inc HL
	ld C, (HL)			; C = Move effect accuracy
	srl C \ srl C \ srl C
	ld B, A
	ld A, C
	or A
	jr z, AttackEffectHit	; effect works if acc == 0
	
	push BC
	ld HL, (AtkPKMN)
	call GetPKMNAbility		; A = pkmn ability
	cp aSereneGrace
	pop BC
	jr nz, AttackEffectNotSereneGrace
	sla C				; if serene grace, then C = C * 2
	
AttackEffectNotSereneGrace:
	push BC
	ld B, 100
	call iRandom			; A = rand, 0 <= A < 100
	pop BC
	cp C
	ret nc				; ret if C <= A
	
AttackEffectHit:
	ld A, B			; restore move effect ID to A
	or A				; return if qNoEffect
	ret z
	
	cp qBlazeKick
	jr nz, AttackEffect_MiscNotBlaze
	ld A, qBurn			; blaze kick has same effect as burn
	
AttackEffect_MiscNotBlaze:
	cp qBlizzard
	jr nz, AttackEffect_MiscNotBlizzard
	ld A, qFreeze			; blizzard has same effect as freeze
	
AttackEffect_MiscNotBlizzard:
	cp qThunder
	jr nz, AttackEffect_MiscNotThunder
	ld A, qParalyze		; thunder has same effect as freeze
	
AttackEffect_MiscNotThunder:
	
	cp qAllStatsRaise + 1
	jr c, AttackEffect_StatRaise
	cp qUserAtkDefLower + 1
	jp c, AttackEffect_StatLower
	cp qYawn + 1
	jp c, AttackEffect_Status
	cp qTorment + 1
	jp c, AttackEffect_VolStatus
	cp qSunnyDay + 1
	jp c, AttackEffect_Field
	cp qFly + 1
	jp c, AttackEffect_Stall
	cp qThunder + 1
	jp c, AttackEffect_Misc
	jp AttackEffect_Spec
	
; ---------------------------------------------------------------
; Deals w/ stat raising move effects
; ---------------------------------------------------------------
	
AttackEffect_StatRaise:
	bit TurnFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_StatRaisePlayer
AttackEffect_StatRaiseOpp:
	ld HL, OppPKMN + PKMN_AtkBoost
	res PKMNFlag, (IY + Asm_Flag1)	; user = opp
	jr AttackEffect_StatRaiseCont2
	
AttackEffect_StatRaisePlayer:
	ld HL, PlayerPKMN + PKMN_AtkBoost
	set PKMNFlag, (IY + Asm_Flag1)	; user = player

AttackEffect_StatRaiseCont2:
	jp AttackEffect_StatRaiseCont
	
; ---------------------------------------------------------------
; Deals w/ stat lowering move effects
; ---------------------------------------------------------------
	
AttackEffect_StatLower:
	bit TurnFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_StatLowerPlayerTurn
AttackEffect_StatLowerOppTurn:
	cp qUserSpAtkLower2			; if (opp turn && lower target)
	jr c, AttackEffect_StatLowerPlayer	; then lower player
	jr AttackEffect_StatLowerOpp2	; elif (opp turn && lower user), then lower opp, skip clear body
	
AttackEffect_StatLowerPlayerTurn:
	cp qUserSpAtkLower2			; if (player turn && lower target)
	jr c, AttackEffect_StatLowerOpp	; then lower opp
	jr AttackEffect_StatLowerPlayer2	; elif (player turn && lower user), then lower player, skip clear body
	
AttackEffect_StatLowerPlayer:		; lower player stats
	ld HL, PlayerPKMN + PKMN_AtkBoost
	set PKMNFlag, (IY + Asm_Flag1)
	jr AttackEffect_StatLowerCont2
	
AttackEffect_StatLowerOpp:			; lower opp stats
	ld HL, OppPKMN + PKMN_AtkBoost
	res PKMNFlag, (IY + Asm_Flag1)
AttackEffect_StatLowerCont2:		; HL = pt to atk boost offset
	call AttackEffect_StatLowerClearBody
	ret z					; ret if clear body
	
	ld D, A
	push HL
	ld BC, PKMN_AtkBoost
	or A
	sbc HL, BC				; HL = pt to pkmn data
	call GetPKMNSub			; A = pkmn sub status, nz = status is not zero
	pop HL
	ld A, D
	jp nz, AttackEffect_ButItFailed	; fail if sub is up
	jr AttackEffect_StatLowerContB	; else, continue
	
AttackEffect_StatLowerPlayer2:		; lower player stats, self-inflicted
	ld HL, PlayerPKMN + PKMN_AtkBoost
	set PKMNFlag, (IY + Asm_Flag1)
	jr AttackEffect_StatLowerContB
	
AttackEffect_StatLowerOpp2:			; lower opp stats, self-inflicted
	ld HL, OppPKMN + PKMN_AtkBoost
	res PKMNFlag, (IY + Asm_Flag1)
AttackEffect_StatLowerContB:		; HL = pt to atk boost offset
	jp AttackEffect_StatLowerCont
	
AttackEffect_StatLowerClearBody:		; subroutine - check target for clear body, input: HL = pt to atk boost, output: z = clear body, save A
	push AF				; save attack effect ID
	push HL				; save pt to pkmn
	ld DE, PKMN_AtkBoost - PKMN_Abil
	or A
	sbc HL, DE
	ld A, (HL)				; A = pkmn ability
	ld (TextAbil), A			; set (TextAbil) to A
	pop HL					; restore pt to pkmn
	
	cp aClearBody				; if ability is clear body
	pop BC
	ld A, B				; restore attack effect ID
	ret nz					; ret w/ nz if not clear body
	
AttackEffect_StatLowerClearBodyDisp:
	ld A, stStat
	ld (TextStat), A			; set (TextStat) to stStat
	rTextbox(tAbilPrevStatLoss)		; then disp msg, return, save all regs, flags
	ret					; ret w/ z if clear body
	
; ---------------------------------------------------------------
; Deals w/ status changing move effects
; ---------------------------------------------------------------

AttackEffect_Status:
	bit TurnFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_StatusPlayerTurn
AttackEffect_StatusOppTurn:
	ld HL, PlayerPKMN
	set PKMNFlag, (IY + Asm_Flag1)	; target is player
	jr AttackEffect_StatusCall
	
AttackEffect_StatusPlayerTurn:
	ld HL, OppPKMN
	res PKMNFlag, (IY + Asm_Flag1)	; target is opp
AttackEffect_StatusCall:			; HL = pt to target pkmn
	ld D, A
	call GetPKMNSub			; HL, DE preserved
	ld A, D
	jr nz, AttackEffect_ButItFailed	; fail if target has a sub
	
	call IsSafeguard			; Both preserve HL
	jr c, AttackEffect_ButItFailed	; //TODO: silence for status side effects
	
	ld BC, PKMN_Status
	add HL, BC
	jp AttackEffect_StatusCont		; HL = pt to status offset
	
AttackEffect_StatusSynchronize:		; subroutine - inflicts status on atk pkmn if sync, input: H = status attack effect ID, DE = pt to pkmn
	ld (TextStat), A			; set (TextStat) to stat condition
	ex DE, HL				; HL = pt to target pkmn data, D = status attack effect ID
	call GetPKMNStatus			; A = pkmn status, saves HL, DE
	ret z					; if pkmn cured of status, then ret
	call GetPKMNAbility			; A = pkmn ability, saves HL, DE
	cp aSynchronize			; if ability is not synchronize
	ret nz					; then ret
	
	ld (TextAbil), A			; set (TextAbil) to aSynchronize
	ld HL, (AtkPKMN)			; else, inflict burn on atk pkmn
	ld BC, PKMN_Status
	add HL, BC				; HL = pt to atk pkmn status
	ld A, (HL)				; A = atk pkmn status
	or A					; if atk pkmn already has status
	ret nz					; then ret
	rTextbox(tAbilStatAtk)		; disp msg 'target's abil stat'ed atk pkmn'
	call FlipPKMNFlag			; TODO: flip back to original state
	ld A, D				; A = status attack effect ID
	jp AttackEffect_Call			; execute status routine, skip safeguard / substitute checks
	
	; TODO: works w/ sub, fails w/ safeguard
	
AttackEffect_StatusChkAbil:			; subroutine - chk if pkmn has abil, input: DE = pt to target, B = ability, C = TextStat, saves HL, DE
	push HL
	ex DE, HL				; HL = pt to target data
	push BC
	call GetPKMNAbility			; A = pkmn ability
	pop BC
	ex DE, HL				; restore DE
	cp B					; compare to input ability
	pop HL					; restore HL
	ret nz					; ret w/ nz if ability doesn't match
	
	ld (TextAbil), A			; set (TextAbil) to ability
	ld A, C				; A = input stat
	ld (TextStat), A			; set (TextStat) to stat
	ret					; ret w/ z, ability matches
	
AttackEffect_StatusPoisonSync:
	push HL
	ld A, stPoison
	ld H, qPoison				; execute routine AE_StatusPoison
	call AttackEffect_StatusSynchronize
	pop HL
	
	ld B, $FF				; use null item ID, no berry that cures poison
	;jr AttackEffect_PKMNBerryCure
	
AttackEffect_PKMNBerryCure:
	call PKMNBerryCure
	ret
	
AttackEffect_StatusSleepChesto:		; checks if chesto berry, and cures sleep
	ld DE, tPKMNChestoBerry
	ld B, iChestoBerry
	jr AttackEffect_PKMNBerryCure
	
AttackEffect_StatusPoisonChkImmunity:	; subroutine - check if pkmn has immunity, input: DE = pt to target
	ld B, aImmunity
	ld C, stPoisoning
	jr AttackEffect_StatusChkAbil
	
AttackEffect_ButItFailed:	
	rTextboxRet(tButItFailed)
	
; ---------------------------------------------------------------
; Deals with volatile statuses
; ---------------------------------------------------------------

AttackEffect_VolStatus:
	bit TurnFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_VolStatusPlayerTurn
AttackEffect_VolStatusOppTurn:
	ld HL, PlayerPKMN
	set PKMNFlag, (IY + Asm_Flag1)	; target is player
	jr AttackEffect_VolStatusCont2
	
AttackEffect_VolStatusPlayerTurn:
	ld HL, OppPKMN
	res PKMNFlag, (IY + Asm_Flag1)	; target is opp
AttackEffect_VolStatusCont2:		; HL = pt to pkmn offset
	jp AttackEffect_VolStatusCont
	
; ---------------------------------------------------------------
; Deals with field effects
; ---------------------------------------------------------------

AttackEffect_Field:
	bit TurnFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_FieldPlayerTurn
AttackEffect_FieldOppTurn:
	ld HL, OppField
	res PKMNFlag, (IY + Asm_Flag1)	; user is opp
	jr AttackEffect_FieldCont2
	
AttackEffect_FieldPlayerTurn:
	ld HL, UserField
	set PKMNFlag, (IY + Asm_Flag1)	; user is player
AttackEffect_FieldCont2:			; HL = pt to weather offset	
	jp AttackEffect_FieldCont
	
; ---------------------------------------------------------------
; Deals with stall move effects
; ---------------------------------------------------------------

AttackEffect_Stall:
	bit TurnFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_StallPlayerTurn
AttackEffect_StallOppTurn:
	ld HL, OppPKMN			; opp turn
	set PKMNFlag, (IY + Asm_Flag1)	; player is target
	jr AttackEffect_StallCont2
	
AttackEffect_StallPlayerTurn:
	ld HL, PlayerPKMN			; player turn
	res PKMNFlag, (IY + Asm_Flag1)	; opp is target
AttackEffect_StallCont2:			; HL = pt to pkmn offset
	ld BC, PKMN_Stall
	add HL, BC				; HL = pt to stall status	
	jp AttackEffect_StallCont
	
; ---------------------------------------------------------------
; Deals with misc effects
; ---------------------------------------------------------------

AttackEffect_Misc:
; fall through to AE_Spec
	
; ---------------------------------------------------------------
; Deals with move-specific effects
; ---------------------------------------------------------------

AttackEffect_Spec:
	bit TurnFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_SpecPlayer
AttackEffect_SpecOpp:
	ld HL, OppPKMN
	ld DE, PlayerPKMN
	res PKMNFlag, (IY + Asm_Flag1)	; user = opp
	jr AttackEffect_SpecCont2
	
AttackEffect_SpecPlayer:
	ld HL, PlayerPKMN
	ld DE, OppPKMN
	set PKMNFlag, (IY + Asm_Flag1)	; user = player
AttackEffect_SpecCont2:			; HL = pt to attacker, DE = pt to target
	jp AttackEffect_SpecCont
	
AttackEffect_BatonPassSwitch:		; subroutine - input: A = team num, DE = pt to pkmn, switches, inits pkmn, w/o resetting status vars
	call BattleInitGet			; returns HL = pt to pkmn in team, saves AF / DE
	
	push DE
	ld BC, BYTES_PER_TPKMN		; Copies basic info
	ldir
	ld (DE), A				; Sets team #
	
	push AF
	xor A					; A = 0
	ld HL, PKMN_Yawn - PKMN_TeamNum
	add HL, DE				; HL = pt to yawn status
	ld (HL), A				; reset yawn status
	ld DE, PKMN_DisableCount - PKMN_Yawn
	add HL, DE
	ld (HL), A				; reset disable count status
	ld DE, PKMN_EncoreCount - PKMN_DisableCount
	add HL, DE
	ld (HL), A				; reset encore count status
	inc HL					; Infatuation status after EncoreCount
	ld (HL), A				; reset infatuation status
	ld DE, PKMN_Torment - PKMN_Infatuation
	add HL, DE
	ld (HL), A				; reset torment status
	ld DE, PKMN_Trap - PKMN_Torment
	add HL, DE
	ld (HL), A				; reset trap status
	pop AF
	pop HL
	
	jp BattleInitReset			; resets status vars (last / encore / disable move), at pkmn at HL, AF not affected
	
AttackEffect_ChkStickyHold:			; subroutine - check if target has sticky hold, input: DE = pt to target pkmn, output: z = sticky hold
	ex DE, HL				; HL = pt to target pkmn
	call GetPKMNAbility			; A = pkmn ability, preserves HL, DE
	ex DE, HL				; DE = pt to target pkmn
	cp aStickyHold
	ret nz					; ret w/ nz if target doesn't have sticky hold
	
	ld (TextAbil), A			; set (TextAbil) to aStickyHold
	ld A, stItemTheft
	ld (TextStat), A			; set (TextStat) to stItemTheft
	rTextbox(tTrgtAbilPrevStat)		; disp 'prev' message, saves flags
	ret					; ret w/ z, ld / Textbox preserve flags
	
AttackEffect_SwitchSub:			; subroutine - determines which pkmn to switch to, perform switch
	or A
	jr nz, AttackEffect_SwitchDoSwitch	; jump to switch if A /= 0
	
AttackEffect_Switch2NotFNT3NotFNT:		; if 2nd pkmn is not FNT and 3rd pkmn is not FNT (A = 0)
	push BC
	ld B, 2
	call iRandom				; 0 <= A < 2
	pop BC
	add A, B
	inc A					; A = 2 | 3 | 5 | 6
	
AttackEffect_SwitchDoSwitch:		; A = (2 - 3) for player | (5 - 6) for opp
	set SilenceFlag, (IY + Asm_Flag2)	; don't display 'player | trainer sent out' msg
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_SwitchDoSwitchOpp
AttackEffect_SwitchDoSwitchPlayer:
	call Turn_SwitchPlayerNoFNT
	call Turn_PriorityPlayer		; HL, DE, (PKMNFlag) changed
	jr AttackEffect_SwitchDoSwitchCont
	
AttackEffect_SwitchDoSwitchOpp:
	call Turn_SwitchOppNoFNT
	call Turn_PriorityOpp		; HL, DE, (PKMNFlag) changed			
	
AttackEffect_SwitchDoSwitchCont:
	res SilenceFlag, (IY + Asm_Flag2)
	push HL
	push DE
	ld HL, tPKMNWasDraggedOut
	call TextboxBlink			; disp msg, w/ blinker
	pop DE
	pop HL
	jp Turn_SwitchCalc			; switch-in ability activation, spikes damage, ret c if all pkmn fainted
	
AttackEffect_FutureDo:
	ld A, (HL)
	or A
	jp nz, AttackEffect_ButItFailed	; fail if future already used
	ld (HL), 3				; otherwise, hit in 3 turns
	inc HL					; HL = pointer to future sp atk
	push HL
	ld HL, (AtkPKMN)
	ld DE, PKMN_SAtk
	add HL, DE
	call LdHLIndRAM
	ex DE, HL				; DE = user sp atk
	pop HL
	ld (HL), E				; load DE into (HL), little-endian
	inc HL
	ld (HL), D
	inc HL					; HL = pointer to future sp def
	push HL
	ld HL, (TargetPKMN)
	ld DE, PKMN_SDef
	add HL, DE
	call LdHLIndRAM
	ex DE, HL				; DE = target sp def
	pop HL
	ld (HL), E				; load DE into (HL), little-endian
	inc HL
	ld (HL), D
	inc HL					; HL = pointer to future hit
	ld A, (HitMiss)
	ld (HL), A				; load A into (HL)
	rTextboxRet(tPKMNForesawAnAttack)
	
AttackEffect_MiscBlaze:
	call AttackEffect_MiscBlazeSub
	ret nz					; ret if target already has status
	call FlipPKMNFlag			; switch from user to target
	ld A, qBurn
	call AttackEffect_StatusCall	; else inflict burn status (AE_StatusBurn)
	jp FlipPKMNFlag
	
AttackEffect_MiscBlazeSub:			; subroutine - returns HL = pt to status, A = status, z = none status
	ex DE, HL				; DE = pt to target pkmn detailed stats
	ld BC, PKMN_Status
	add HL, BC
	ld A, (HL)
	or A
	ret
	
AttackEffect_MiscBlizzard:
	call AttackEffect_MiscBlazeSub
	ret nz					; ret if target already has status
	call FlipPKMNFlag
	call AttackEffect_StatusFreeze	; else inflict freeze status
	jp FlipPKMNFlag
	
; ---------------------------------------------------------------
; Continuation point
; ---------------------------------------------------------------

AttackEffect_StatRaiseCont:			; HL = pt to atk boost offset		
	
AttackEffect_StatLowerCont:			; HL = pt to atk boost offset

AttackEffect_StatusCont:			; HL = pt to status offset

AttackEffect_VolStatusCont:			; HL = pt to pkmn offset

AttackEffect_FieldCont:			; HL = pt to weather offset

AttackEffect_StallCont:			; HL = pt to stall status

AttackEffect_MiscCont:			; HL = pt to atk pkmn, DE = pt to def pkmn

AttackEffect_SpecCont:			; HL = pt to atk pkmn, DE = pt to def pkmn

AttackEffect_Call:				; A = move effect ID, should not be 0
	ld BC, RCallRet
	push BC				; push return address on top of stack
	
	push HL				; save HL
	push DE				; save DE
	push AF				; save AF
	ld HL, AEPg				; HL = pt to start of attack effect pages table
AttackEffect_PageLp:
	cp (HL)				; if move effect ID < AEPg[3 * i]
	inc HL					; move HL to pt to page address, AEPg[3 * i + 1], flags not affected
	jr c, AttackEffect_Page		; then break
	inc HL \ inc HL			; else, HL += 2, move to next table entry
	jr AttackEffect_PageLp		; loop until page found
	
AttackEffect_Page:
	call LdHLIndRAM			; HL = (HL), pt to start of page
	push HL
	pop BC					; BC = pt to start of page
	call StackPushBC			; push lbl into stack, A not changed
	call RCallCopy			; copy routine at BC into SwapZone, A preserved
	
	pop AF					; get AF
	dec A					; A = A - 1, A < 128
	add A, A				; A = A * 2
	ld HL, AEDb				; HL = pt to start of attack effect database
	ld D, 0				; inline GetTable2
	ld E, A				; DE = zero extended A
	add HL, DE				; HL = pt to table entry
	call LdHLIndRAM			; HL = pt to location of attack effect routine in page
	push HL
	pop IX					; IX = pt to location of attack effect routine in page

	pop DE					; get DE
	pop HL					; get HL
	
	jp (IX)				; execute move effect routine
	
AttackEffect_CallSilent:			; subroutine - set silence flag before call
	set SilenceFlag, (IY + Asm_Flag2)
	call AttackEffect_Call
	res SilenceFlag, (IY + Asm_Flag2)
	ret
	