; ===============================================================
; Info
; ===============================================================

; ---------------------------------------------------------------
; Info:
; Browse poke info, and return keypress
; Input: HL = poke ID or pt to poke data
; Output: A = Exit mode
;	     = 0 (Exit)
;	     = 1 (Up)
;	     = 2 (Down)
; ---------------------------------------------------------------

Info:
	xor A				; init page = Info
	ld (InfoPage), A
	
Info2:
	push HL			; Draw UI, header, profile
	call InfoUI
	call InfoHeader
	pop HL
	call InfoProfile
	
	ld A, (InfoPage)
	or A
	jp z, InfoPgInfo
	cp 1
	jp z, InfoPgSkills
	cp 2
	jp z, InfoPgMoves
	
InfoLp:					; expects HL to be preserved
	push HL
	RCall( cGetInput ) ;b_call(_GetCSC)
	pop HL
	or A
	jr z, InfoLp
	
	cp gkEnter
	jr z, InfoA
	cp gk2nd
	jr z, InfoA
	cp gkMode
	jp z, PartyQuit ; Quit
	cp gkAlpha
	jr z, InfoRet
	cp gkClear
	jr z, InfoRet
	cp gkLeft
	jr z, InfoLeft
	cp gkRight
	jr z, InfoRight
	cp gkUp
	jr z, InfoUp
	cp gkDown
	jr z, InfoDown
	
	jr InfoLp
	
InfoLeft:
	ld A, (InfoPage)
	or A
	jr z, InfoLp
	dec A
	ld (InfoPage), A
	jr Info2
	
InfoRight:
	ld A, (InfoPage)
	cp 2
	jr z, InfoLp
	inc A
	ld (InfoPage), A
	jr Info2
	
InfoUp:
	ld A, 1
	ret
	
InfoDown:
	ld A, 2
	ret
	
InfoRet:
	xor A
	ret
	
InfoA:
	ld A, (InfoPage)			; If Pg != 2
	cp 2					; Return
	jr nz, InfoRet			; Else...
	
	xor A					; Init move = 0 (first move)
	ld (InfoMoveNum), A
InfoA2:
	push HL
	RCall( cGetMove )			; HL = Move1 + (MoveNum)
	ld A, (InfoMoveNum)
	ld B, 0
	ld C, A
	add HL, BC
	call GetByte
	call InfoPgMoveInfo
	pop HL
InfoALp:
	push HL
	RCall( cGetInput ) ;b_call(_GetCSC)
	pop HL
	or A
	jr z, InfoALp
	
	cp gkEnter
	jp z, Info2
	cp gk2nd
	jp z, Info2
	cp gkMode
	jp z, PartyQuit ; Quit
	cp gkClear
	jp z, Info2
	cp gkAlpha
	jp z, Info2
	cp gkUp
	jr z, InfoAUp
	cp gkDown
	jr z, InfoADown
	
InfoAUp:
	ld A, (InfoMoveNum)
	or A
	jr z, InfoALp
	dec A
	ld (InfoMoveNum), A
	jr InfoA2
	
InfoADown:
	ld A, (InfoMoveNum)
	cp 3
	jr z, InfoALp
	inc A
	ld (InfoMoveNum), A
	jr InfoA2
		
; ---------------------------------------------------------------
; InfoPgInfo
; Writes info, then returns to main lp
; ---------------------------------------------------------------

InfoPgInfo:
	di
	push HL
	rDrawBoxLbl(33, 9, XX)	; Type txtbox
	rHorizLineB(33, 24, 63)	; separator
	rDrawBoxLbl(33, 25, XX)	; Ability txtbox
	rVertLineD(33, 40, 8)	; ability flavor txtbox
	rVertLineD(95, 40, 8)
	rHorizLineD(34, 47, 61)
	rDrawRect(34, 32, 61, 7)	; background fill
	rHorizLineB(33, 48, 63)	; separator
	rDrawBoxLbl(33, 49, XX)		; Nature txtbox
	
	set TextInverse, (IY + TextFlags)	; Text
	rVPutS(40, 9, tType)
	rCopyBuffSect(40, 10, 15, 5)
	rFillBuffSectL(WFill, 40, 9, 16, 6)
	rVPutS(40, 25, tAbility)
	rCopyBuffSect(40, 26, 21, 5)
	rFillBuffSectL(WFill, 40, 25, 24, 6)
	rVPutS(40, 49, tNature)
	rCopyBuffSect(40, 50, 22, 5)
	rFillBuffSectL(WFill, 40, 49, 24, 6)
	res TextInverse, (IY + TextFlags)
	pop HL
	
	push HL			; Disp Types
	RCall( cGetType )
	push HL
	call GetByte			; first type --> A
	push AF
	ld HL, 36 * 256 + 16		; (36, 16)
	RCall( cDispType )
	pop AF
	pop HL
	inc HL
	call GetByteC			; second type --> C
	cp C
	jr z, InfoPgInfoSkipType	; skip if same as 1st
	ld A, C
	ld HL, 66 * 256 + 16
	RCall( cDispType )
InfoPgInfoSkipType:
	pop HL
	
	push HL			; Disp Ability
	set TextInverse, (IY + TextFlags)
	RCall( cGetAbility )
	push AF
	RCall( cGetAbilityText )
	rTextCoord(35, 32)
	RCall( cArcText )
	res TextInverse, (IY + TextFlags)
	pop AF
	
	RCall( cGetAbilityDesc )	; Disp Ability description
	rTextCoord(35, 40)
	RCall( cArcText )
	rCopyBuffSect(34, 40, 61, 7)
	pop HL
	
	push HL			; Disp Nature
	RCall( cGetNature )
	RCall( cGetNatureText )
	rTextCoord(35, 56)
	RCall( cArcText )
	pop HL
	
	ei
	jp InfoLp
	
; ---------------------------------------------------------------
; InfoPgSkills
; Writes skills, then returns to main lp
; ---------------------------------------------------------------

InfoPgSkills:
	di
	push HL
	rDrawBoxLbl(33, 9, XX)		; item box
	rHorizLineB(33, 24, 63)		; separator
	rDrawRectD(33, 25, 63, 7)		; stats box
	rVertLineD(33, 32, 22)
	rVertLineD(95, 32, 22)
	rHorizLineD(34, 53, 61)
	rDrawRect(34, 32, 12, 21)		; stats background fill
	rDrawRect(59, 32, 23, 21)
	rHorizLineB(33, 54, 63)		; separator
	
	set TextInverse, (IY + TextFlags)
	rVPutS(40, 9, tItem)		; 'Item'
	rCopyBuffSect(40, 9, 17, 6)
	rFillBuffSectL(WFill, 40, 9, 24, 6)
	rVPutS(40, 25, tStats)		; 'Stats'
	rCopyBuffSect(40, 25, 17, 6)
	rFillBuffSectL(WFill, 40, 25, 24, 6)
	
	rVPutS(36, 32, tHP)			; Stat text
	rVPutS(35, 39, tAtk)
	rVPutS(35, 46, tDef)
	rVPutS(60, 32, tSpAtk)
	rVPutS(60, 39, tSpDef)
	rVPutS(61, 46, tSpeed)
	res TextInverse, (IY + TextFlags)
	pop HL
	
	push HL
	bit InfoFlag, (IY + Asm_Flag3)	; if disp info w/ data
	jr nz, InfoPgSKillsItem		; then get A = current item
	RCall( cGetItem )			; else get A = item from poke data
	jr InfoPgSkillCont
	
InfoPgSkillsItem:
	ld BC, PKMN_Item
	add HL, BC
	ld A, (HL)
	
InfoPgSkillCont:
	RCall( cGetItemText )
	rTextCoord(35, 16)
	RCall( cArcText )
	rCopyBuffSect(35, 17, 59, 5)
	pop HL
	
	push HL
	rTextCoord(47, 32)
	xor A
InfoPgSkillsLp:
	push HL
	push AF
	RCall( cCalcStat )		; disp stat
	RCall( cIntStr )
	b_call(_VPutS)
	pop AF
	inc A
	
	push AF			; position text
	ld HL, InfoPgSkillsDb	; excluding HP, since already disp
	dec A
	RCall( cGetTable2 )
	ld (PenCol), HL
	pop AF
	pop HL
	cp 6
	jr nz, InfoPgSkillsLp
	
	rCopyBuffSect(47, 33, 11, 19)
	rCopyBuffSect(83, 33, 11, 19)
	pop HL
	
	bit InfoFlag, (IY + Asm_Flag3)
	jr z, InfoPgSkillSkipHP	; skip if not displaying info w/ data
	push HL
	rVPutS(34, 56, tCurHP)	; 'Cur. HP:'
	pop HL
	push HL
	rTextCoord(60, 56)
	ld BC, PKMN_HP
	add HL, BC
	call LdHLIndRAM		; HL = current HP
	RCall( cIntStr )		; HL = pt to HP as string
	b_call(_VPutS)		; display current HP
	ld A, '/'
	b_call(_VPutMap)		; display '/'
	pop HL
	push HL
	xor A
	RCall( cCalcStat )		; HL = max HP stat
	RCall( cIntStr )		; HL = pt to max HP as string
	b_call(_VPutS)		; display max HP
	pop HL
	
InfoPgSkillSkipHP:
	ei
	jp InfoLp
	
InfoPgSkillsDb:
.db 47, 39, 47, 46
.db 83, 32, 83, 39, 83, 46
	
; ---------------------------------------------------------------
; InfoPgMoves
; Writes moves, then returns to main lp
; ---------------------------------------------------------------

InfoPgMoves:
	di
	push HL
	rHorizLineD(33, 9, 63)	; move box outline
	rVertLineD(33, 9, 30)
	rVertLineD(95, 9, 30)
	rHorizLineD(33, 38, 63)
	rDrawRect(34, 10, 61, 28)	; move box fill
	pop HL
	
	push HL
	set TextInverse, (IY + TextFlags)
	rTextCoord(35, 10)
	RCall( cGetMove )
	ld B, 4
InfoPgMovesLp:
	push HL
	push BC
	call GetByte
	RCall( cGetMoveText )
	RCall( cArcText )
	pop BC
	pop HL
	ld A, 35
	ld (PenCol), A
	ld A, (PenRow)
	add A, 7
	ld (PenRow), A
	inc HL
	djnz InfoPgMovesLp
	res TextInverse, (IY + TextFlags)
	pop HL
	
	ei
	jp InfoLp
	
; ---------------------------------------------------------------
; InfoPgMoveInfo
; Writes info about move (Input = A), then ret
; Draws box around selected move (Input = (InfoMoveNum))
; ---------------------------------------------------------------

InfoPgMoveInfo:
	di
	push AF
	rVertLineD(33, 39, 25)		; boxes
	rVertLineD(95, 39, 25)
	rHorizLineD(34, 49, 61)
	rHorizLineD(34, 63, 61)
	
	res LineColor, (IY + Asm_Flag1)	; white on gray buffer
	rHorizLine(1, 49, 30)
	set LineColor, (IY + Asm_Flag1)
	rHorizLineD(1, 49, 30)
	rDrawRect(1, 50, 17, 13)		; power/acc background fills
	rDrawRectW(18, 50, 13, 13)
	
	set TextInverse, (IY + TextFlags)	; Text - power and accuracy
	rVPutS(2, 50, tPOW)
	rVPutS(2, 56, tACC)
	res TextInverse, (IY + TextFlags)
	
	rDrawRectW3(35, 40, 59, 7)		; Clear out location of type and PP
	pop AF
	
	push AF
	push AF
	RCall( cGetMoveType )		; get Type
	ld HL, 35 * 256 + 40		; (35, 40)
	RCall( cDispType )
	
	rVPutS(66, 40, tPP)		; PP Text
	ld A, ' '
	b_call(_VPutMap)
	pop AF
	push AF
	RCall( cGetMovePP)		; get Max PP --> HL
	ld L, A
	ld H, 0
	bit InfoFlag, (IY + Asm_Flag3)
	jr z, InfoPageMoveInfoPP
	
	push HL			; save max PP
	call PartyGetPKMN		; HL = pt to data of selected pkmn in party
	ld BC, PKMN_PP
	add HL, BC			; HL = pt to PP array
	ld A, (InfoMoveNum)
	ld C, A			; C = current move num, B = 0
	add HL, BC			; HL = pt to PP of current move
	ld A, (HL)
	ld H, B
	ld L, A			; HL = zero extended A
	call InfoPageMoveInfoSlash	; display PP, display '/'
	pop HL				; get max PP
	RCall( cIntStr )		; HL = pt to max PP as string
	jr InfoPageMoveInfoMaxPP
	
InfoPageMoveInfoPP:
	call InfoPageMoveInfoSlash	; display max PP, slash

InfoPageMoveInfoMaxPP:
	b_call(_VPutS)
	rCopyBuffSect(66, 41, 28, 5)
	pop AF
	
	push AF
	rTextCoord(19, 50)
	RCall( cGetMovePow )		; Power --> A
	or A
	jr nz, InfoPgMoveInfoN0
	ld HL, t3Hyphen
	b_call(_VPutS)
	jr InfoPgMoveInfo0
InfoPgMoveInfoN0:
	ld L, A
	ld H, 0
	RCall( cIntStr )
	b_call(_VPutS)
InfoPgMoveInfo0:
	pop AF
	rTextCoord(19, 56)
	RCall( cGetMoveAcc )		; Acc --> A
	or A
	jr nz, InfoPgMoveInfo2N0
	ld HL, t3Hyphen
	b_call(_VPutS)
	jr InfoPgMoveInfo20
InfoPgMoveInfo2N0:
	ld L, A
	ld H, 0
	RCall( cIntStr )
	b_call(_VPutS)
InfoPgMoveInfo20:
	rCopyBuffSect(19, 51, 11, 11)
	
	rDrawRectW2(34, 10, 61, 28)	; Clear out background for selecting box
	rDrawRectW3(34, 50, 61, 13)	; Clear out background for move description
	ld A, (InfoMoveNum)
	ld H, A			; A = Num * 7 + 10
	ld L, 7			;   = 10, 17, 24, 31
	call MultHL			; Y coordinate
	ld A, 10
	add A, L
	ld L, A
	ld H, 34
	ld B, 61
	call HorizLineDark		; rHorizLine(34, Y, 61)
	ld B, 7
	call VertLineDark		; rVertLine(34, Y, 7)
	ld H, 94
	call VertLineDark		; rVertLine(94, Y, 7)
	ld A, 6
	add A, L
	ld L, A
	ld H, 34
	ld B, 61
	call HorizLineDark		; rHorizLine(34, Y+6, 61)
	pop AF
	
	push AF
	ld HL, TextSize
	ld (HL), 0			; size limit is none
	RCall( cGetMoveDesc )	; get HL = pt to move description
	RCall( cArcTextDc )		; decode text, returns HL = pt to decoded text
	push HL
	ld B, 255
	RCall( cStringLen )
	pop HL
	ld A, C
	cp 60				; if len(str) < 60
	jr c, InfoPgMoveInfo1Line	; disp single line
	
	push HL			; else disp 2 lines
	xor A				; find first line
	ld B, A			; B = pos at last ' '
	ld C, A			; C = current pixel len
	ld D, A			; D = current pos
InfoPgMoveInfoChar0:
	ld B, D
InfoPgMoveInfoChar:
	ld A, (HL)			; get char, update pos
	or A
	jr z, InfoPgMoveInfoSkip
	inc HL
	inc D
	push AF
	push BC
	push DE
	push HL
	;RCall( cCharLen )
	ld H, 0			; inline CharLen
	ld L, A
	add HL, HL			; HL * 8
	add HL, HL
	add HL, HL
	b_call(_SFont_Len)
	ld A, B			; A = char length in pixels
	pop HL
	pop DE
	pop BC
	add A, C
	ld C, A			; update current pixel len
	pop AF
	
InfoPgMoveInfoSkip:
	cp ' '
	jr z, InfoPgMoveInfoEnd
	or A
	jr z, InfoPgMoveInfoEnd
	jr InfoPgMoveInfoChar
	
InfoPgMoveInfoEnd:
	ld A, C
	cp 60				; if not reached 60 pixels
	jr c, InfoPgMoveInfoChar0	; then update pos at last ' '
	pop HL				; else return B = cutoff pos
	
	push HL
	push BC
	ld A, 35
	ld (PenCol), A
	ld A, 50
	ld (PenRow), A
	RCall( cPutSN )
	pop BC
	pop HL
	ld D, 0
	ld E, B
	add HL, DE
	ld B, 40
	ld A, 35
	ld (PenCol), A
	ld A, 56
	ld (PenRow), A
	RCall( cPutSN )
	jr InfoPgMoveInfoCopy
	
InfoPgMoveInfo1Line:
	ld A, 35
	ld (PenCol), A
	ld A, 50
	ld (PenRow), A
	b_call(_VPutS)
	
InfoPgMoveInfoCopy:
	ld A, (PenCol)
	cp 75
	jr c, InfoPgMoveInfoBound
	ld A, 35
	ld (PenCol), A
	ld A, 56
	ld (PenRow), A
InfoPgMoveInfoBound:
	pop AF
	RCall( cGetMoveEffAcc )		; get move effect acc
	or A
	jr z, InfoPgMoveInfoA0
	push AF
	ld A, ' '
	b_call(_VPutMap)
	ld A, '('
	b_call(_VPutMap)
	pop AF
	ld H, 0
	ld L, A
	RCall( cIntStr )
	b_call(_VPutS)
	ld A, '%'
	b_call(_VPutMap)
	ld A, ')'
	b_call(_VPutMap)
	
InfoPgMoveInfoA0:
	rCopyBuffSect(34, 50, 61, 13)
	
	ei
	ret

InfoPageMoveInfoSlash:			; subroutine - display current value of HL, displays slash, saves HL (string pt)
	RCall( cIntStr )
	push HL
	b_call(_VPutS)
	ld A, '/'
	b_call(_VPutMap)
	pop HL
	ret
	
; ---------------------------------------------------------------
; InfoUI:
; Draws background UI; same for all pages
; ---------------------------------------------------------------
	
InfoUI:
	di
	RCall( cClearBuff )
	
	rHorizLineB(0, 6, 96)	; top line
	
	rHorizLineD(0, 7, 96)	; left box
	rVertLineD(0, 7, 57)
	rVertLineD(31, 7, 57)
	rHorizLineD(0, 63, 32)
	rDrawRect(1, 8, 30, 55)
	
	rHorizLineB(32, 8, 64)	; right frame
	rVertLineB(32, 8, 56)
	ei
	ret
	
; ---------------------------------------------------------------
; InfoHeader:
; Draws header; different in each page
; Input: (InfoPage)
; ---------------------------------------------------------------

InfoHeader:
	di
	rDrawRectD(0, 0, 39, 6)
	
	ld A, (InfoPage)
	ld H, A
	ld L, 26
	call MultHL
	ld DE, sHeaderA
	add HL, DE
	push HL
	pop IX
	ld A, 39
	ld L, 0
	RCall( cgsLargeSprite )
	ei
	
	di
	set TextInverse, (IY + TextFlags)
	rVPutS(1, 0, tPKMN)
	ld A, ' '
	b_call(_VPutMap)
	
	ld B, 36
	ld HL, tMoves			; If (InfoPage) == 0:
	ld A, (InfoPage)		;	HL = tInfo
	or A				; Elif (InfoPage) == 1:
	jr nz, InfoHeaderN0		;	HL = tSkills
	ld B, 32
	ld HL, tInfo			; Else:	# (InfoPage) == 2
InfoHeaderN0:				;	HL = tMoves
	cp 1
	jr nz, InfoHeaderN1
	ld HL, tSkills
InfoHeaderN1:
	push BC
	b_call(_VPutS)
	pop BC
	res TextInverse, (IY + TextFlags)
	
	ld A, 1
	ld L, 0
	ld C, 6
	RCall( cCopyBuffSect )		; in if blocks, leave a len = x
	rDrawRectW(0, 0, 39, 6)	; for param of copy
	ei
	ret
	
; ---------------------------------------------------------------
; Draws poke profile; different with each poke
; Input: HL = pointer to poke data
; ---------------------------------------------------------------

InfoProfile:
	di
	push HL
	RCall( cGetPokeID )
	rArcPKMN(4, 11)
	pop HL
	ei
	
	di
	push HL
	RCall( cGetName )
	call InfoProfileName
	
	set TextInverse, (IY + TextFlags)
	rVPutS(2, 49, tLv100)
	res TextInverse, (IY + TextFlags)
	pop HL
	
	push HL
	ld BC, 23 * 256 + 49		; (23, 49)
	RCall( cDispGender )
	pop HL
	ei
	
	bit InfoFlag, (IY + Asm_Flag3)
	ret z				; ret, skip disp status, if InfoFlag is not set
	
	push HL
	ld BC, PKMN_Status
	add HL, BC
	ld A, (HL)			; A = pkmn status
	ld HL, 2 * 256 + 56		; (2, 56)
	set TextInverse, (IY + TextFlags)
	RCall( cDispStatusX )		; display status at arbitrary location
	res TextInverse, (IY + TextFlags)
	pop HL
	ret
	
InfoProfileName:			; sub-subroutine: disps name, input: HL = pt to name
	ld A, 10
	ld (TextSize), A		; set size limit to 10
	RCall( cArcTextDc )		; decode text, return HL = pt to decoded text
	rTextCoord(2, 36)
	ld B, 10			; max len = 10 chars
InfoProfileNameLp:
	ld A, (HL)
	or A
	ret z
	set TextInverse, (IY + TextFlags)
	b_call(_VPutMap)
	res TextInverse, (IY + TextFlags)
	inc HL
	ld A, (PenCol)
	cp 25
	jr c, InfoProfileNameNotNew	; moves to newline if x >= 25
	rTextCoord(2, 42)
InfoProfileNameNotNew:
	djnz InfoProfileNameLp
	ei
	ret