; ===============================================================
; Party
; ===============================================================

; ---------------------------------------------------------------
; Party:
; Brings up party menu, lets user shift pkmn and see summary
; Output: (PartySwitch) = pkmn to switch to (0 = none)
; ---------------------------------------------------------------

Party:
	ld HL, PlayerPKMN
	ld DE, TeamPKMN1
	ld BC, BYTES_PER_TPKMN
	ldir					; copy to team data
	
	xor A
	ld (PartySel), A			; set (PartySel) to 0, first pkmn
	ld (PartySwitch), A			; initially, no pkmn switch
	
PartyDraw:					; draws UI + text
	call PartyDrawSub
	
PartyLp:
	RCall( cGetInput )
	or A
	jr z, PartyLp
	cp gkEnter
	jp z, PartyOpt
	cp gk2nd
	jp z, PartyOpt
	cp gkClear
	jr z, PartyRet
	cp gkAlpha
	jr z, PartyRet
	cp gkMode
	jr z, PartyQuit
	cp gkUp
	jr z, PartyUp
	cp gkDown
	jr z, PartyDown
	cp gkLeft
	jr z, PartyLeft
	cp gkRight
	jr z, PartyRight
	
	jr PartyLp
	
PartyRet:
	bit SwitchFlag, (IY + Asm_Flag2)	;//TODO: fix
	jr nz, PartyLp			; if switch after faint, then can't cancel
	ret
	
PartyQuit:					; same as Battle_Quit, for Overworld code
	xor A
	ld (PKMNSaveCurrent), A		; invalidate current save
	jp Quit
	
PartyUp:
	di
	ld A, (PartySel)
	ld (PartyPrevSel), A
	call PartyClose			; close pokeball at (PartySel), A preserved
	or A
	jr z, PartyUpOver			; jump to overflow if sel first pkmn
	dec A
	jr PartyUpCont
		
PartyUpOver:
	ld A, 3
	
PartyUpCont:
	call PartyOpen			; open pokeball at (PartySel) - 1, A preserved
	ld (PartySel), A
	ei
	jr PartyLp
	
PartyDown:
	di
	ld A, (PartySel)
	ld (PartyPrevSel), A
	call PartyClose
	cp 3
	jr z, PartyDownOver			; jump to overflow if sel cancel option
	inc A
	jr PartyDownCont
	
PartyDownOver:
	xor A					; A = 0
	
PartyDownCont:
	call PartyOpen
	ld (PartySel), A
	ei
	jr PartyLp
	
PartyLeft:
	ld A, (PartySel)
	cp 3
	jr nz, PartyLp
	
	di
	call PartyClose			; A = 3
	ld A, (PartyPrevSel)
	call PartyOpen
	ld (PartySel), A
	ei
	jp PartyLp
	
PartyRight:
	ld A, (PartySel)
	cp 3
	jp z, PartyLp
	
	di
	ld (PartyPrevSel), A
	call PartyClose
	ld A, 3
	call PartyOpen
	ld (PartySel), A
	ei
	jp PartyLp
		
PartyOpt:
	ld A, (PartySel)
	cp 3
	ret z					; if opt = 3 (cancel), then return
	
	xor A
	ld (PartyOptSel), A
	
PartyOptDraw:
	di
	rDrawRectW3(1, 49, 58, 14)		; clear text area
	rArcText(2, 49, tDoWhatWith)	; disp 'Do what with
	rArcText(2, 56, tThisPKMN)		;       this PKMN?'
	rCopyBuffSect(1, 49, 58, 14)	; copy to buffer 2
	
	rDrawRectW3(61, 55, 35, 9)		; clear 'cancel' area
	rDrawBoxOut(61, 41, 35, 23)		; draw box outline
	rArcText(63, 42, tShift)		; disp 'Shift'
	rArcText(63, 49, tSummary)		; disp 'Summary'
	rArcText(63, 56, tCancel)		; disp 'Cancel'
	rCopyBuffSect(62, 42, 33, 21)	; copy to buffer 2
	ei
	
	ld A, (PartyOptSel)
	call PartyOptFill			; fill in first option
	
PartyOptLp:
	RCall( cGetInput )
	or A
	jr z, PartyOptLp
	
	halt \ halt \ halt
	cp gkEnter
	jr z, PartyOptA
	cp gk2nd
	jr z, PartyOptA
	cp gkClear
	jr z, PartyOptCancel
	cp gkAlpha
	jr z, PartyOptCancel
	cp gkMode
	jp z, PartyQuit
	cp gkUp
	jr z, PartyOptUp
	cp gkDown
	jr z, PartyOptDown
	jr PartyOptLp
	
PartyOptUp:
	di
	ld A, (PartyOptSel)
	call PartyOptClear
	or A
	jr z, PartyOptUpOver
	dec A
	jr PartyOptUpCont
	
PartyOptUpOver:
	ld A, 2
PartyOptUpCont:
	call PartyOptFill
	ld (PartyOptSel), A
	ei
	jr PartyOptLp
	
PartyOptDown:
	di
	ld A, (PartyOptSel)
	call PartyOptClear
	cp 2
	jr z, PartyOptDownOver
	inc A
	jr PartyOptDownCont
	
PartyOptDownOver:
	xor A
PartyOptDownCont:
	call PartyOptFill
	ld (PartyOptSel), A
	ei
	jr PartyOptLp
	
PartyOptCancel:
	jp PartyDraw
	
PartyOptA:
	ld A, (PartyOptSel)
	cp 2					; if Sel == 2
	jr z, PartyOptCancel			; then Cancel
	or A					; if Sel /= 0 (must be 1)
	jp nz, PartyOptSummary		; then Summary
						; else Shift (must be 0)
PartyOptShift:
	ld A, (PartySel)			; 0 <= A <= 2 (can't be 3, cancel)
	call PartyGetPKMN			; HL = pt to selected pkmn
	ld BC, PKMN_Status			; inline GetPKMNStatus
	add HL, BC
	ld A, (HL)				; A = pmkn status
	cp stFNT
	jr nz, PartyOptShiftNotFNT
	
	ld HL, tHas				; disp msg
	ld DE, tNoEnergyLeft
	call PartyDisp
	jp PartyOptDraw
	
PartyOptShiftNotFNT:
	ld A, (PartySel)
	or A					; if A = 0, then disp 'PKMN is still in battle!'
	jr nz, PartyOptShiftDiff		; else jump
	
	ld HL, tIs				; disp msg
	ld DE, tStillInBattle
	call PartyDisp
	jp PartyOptDraw
	
PartyOptShiftDiff:
	ld A, (PlayerPKMN + PKMN_Trap)	; A = trap status (0 if no, 1 if last turn, else turn count)
	or A
	jr nz, PartyOptShiftCantEscape
	
	ld A, (OppPKMN + PKMN_Abil)		; A = ability of opp pkmn
	or A					; aArenaTrap = 0, cp aArenaTrap
	jr z, PartyOptShiftIsTrapped
	
	ld A, (OppPKMN + PKMN_PrevEscape)	; A = prev escape status (1 = yes, 0 = no)
	or A
	jr z, PartyOptShiftCanEscape
	
PartyOptShiftCantEscape:
	bit SwitchFlag, (IY + Asm_Flag2)
	jr nz, PartyOptShiftCanEscape	; can escape, if switching after faint
	ld HL, tCantEscape			; if A = 1, then disp 'Can't escape!'
	call PartyDisp3
	jp PartyOptDraw
	
PartyOptShiftIsTrapped:
	bit SwitchFlag, (IY + Asm_Flag2)
	jr nz, PartyOptShiftCanEscape	; can escape, if switching after faint
	ld A, (PlayerPKMN + PKMN_Abil)
	cp aLevitate
	jr z, PartyOptShiftCanEscape	; can escape, if levitate
	
	ld HL, PlayerPKMN
	ld E, eFlying
	call PKMNIsType
	jr z, PartyOptShiftCanEscape	; can escape, if flying type
	
	ld HL, tEmpty
	ld DE, tIsTrapped
	call PartyDisp0			; disp msg, saves BC
	jp PartyOptDraw			; restore (PartySel)
	
PartyOptShiftCanEscape:
	ld A, (PlayerPKMN + PKMN_Ingrain)	; A = ingrain status (0 if no, 1 if yes)
	or A
	jr z, PartyOptShiftNoIngrain
	
	ld HL, tEmpty 
	ld DE, tAnchoredItself
	call PartyDisp0			; disp msg, saves BC
	jp PartyOptDraw
	
PartyOptShiftNoIngrain:
	ld A, (PartySel)
	ld B, A
	ld A, $FF
	sub B					; A = $FF - (PartySel), $FF | $FE | $FD
	ld (PartySwitch), A
	ret
	
PartyOptSummary:
	xor A				; init page = Info
	ld (InfoPage), A
	set InfoFlag, (IY + Asm_Flag3)
	
PartyOptSummaryLp:
	call PartyGetPKMN
	call Info2				; bring up info page on pkmn
	
	or A
	jr z, PartyOptSummary0
	cp 1
	jr z, PartyOptSummary1	; If Mode == Down
	
	ld B, 2			; B = 2, 3 is max num pkmn in party
	ld A, (PartySel)		; move down
	cp B				; only if A /= (SelectMax)
	jr z, PartyOptSummaryLp
	inc A
	ld (PartySel), A
	jr PartyOptSummaryLp
	; TODO: when A = 0 or 2, skip redraw
	
PartyOptSummary1:
	ld A, (PartySel)		; Elif Mode == Up
	or A				; move up
	jr z, PartyOptSummaryLp
	dec A
	ld (PartySel), A
	jr PartyOptSummaryLp
	
PartyOptSummary0:
	res InfoFlag, (IY + Asm_Flag3)
	call PartyDrawSub
	jp PartyOptDraw
	
PartyOptFill:					; subroutine - fills option rectangle, by clear buffer 1 + invert buffer 2, preserves A
	push AF
	call PartyOptGetY			; L = Y coordinate
	ld H, 62
	ld BC, 33 * 256 + 7
	push HL
	push BC
	RCall( cDrawRectWhite )		; draw white rectangle on buffer 1
	pop BC
	pop HL
	RCall( cInvRectDark )		; invert rectangle area on buffer 2, same coords
	pop AF
	ret
	
PartyOptClear:				; subroutine - clears option rectangle, by invert buffer 2 + copy to buffer 1, preserves A
	push AF
	call PartyOptGetY
	ld H, 62
	ld BC, 33 * 256 + 7
	push HL
	push BC
	RCall( cInvRectDark )		; invert rectangle area on buffer 2
	pop BC
	pop HL
	ld A, H				; A = x parameter
	RCall( cCopyBuffSect2 )		; copy buffer 2 to buffer 1
	pop AF
	ret
	
PartyOptGetY:					; subroutine - input: A = opt selection (0, 1, 2), output: L = Y coordinate (42, 49, 56)
	ld H, A
	ld L, 7
	call MultHL
	ld A, 42
	add A, L				; A = (A * 7) + 42
	ld L, A
	ret
	
PartyDrawSub:					; subroutine - draws pkmn boxes, text, name/species
	di
	RCall( cClearBuff )
	
	rDrawBoxOut(10, 1,  50, 16)		; draw pkmn boxes
	rDrawBoxOut(10, 16, 50, 16)
	rDrawBoxOut(10, 31, 50, 16)
	
	rDrawBoxOut(0, 48, 60, 16)		; draw prompt box
	rDrawBoxOut(61, 55, 35, 9)		; draw 'cancel' box
	
	rGSLargeSprite(0, 4,  sPokeballC)
	rGSLargeSprite(0, 19, sPokeballC)
	rGSLargeSprite(0, 34, sPokeballC)
	ei
	
	di
	rArcText(2, 49, tChooseAPKMN)	; 'Choose a PKMN' text
	rCopyBuffSect(1, 49, 58, 7)
	
	rArcText(63, 56, tCancel)		; 'Cancel' text
	rCopyBuffSect(62, 56, 33, 7)
	ei
	
	di
	ld A, (TeamPKMN1 + PKMN_Species)	; A = species of first pkmn
	ld HL, TeamPKMN1
	ld C, 10
	rTextCoord(12, 2)			; disp name at (12, 2), A, HL not affected
	call PartyName	
	rCopyBuffSect(11, 2, 48, 8)
	
	ld A, (TeamPKMN2 + PKMN_Species)	; A = species of second pkmn
	ld HL, TeamPKMN2
	ld C, 25
	rTextCoord(12, 17)
	call PartyName
	rCopyBuffSect(11, 17, 48, 8)
	
	ld A, (TeamPKMN3 + PKMN_Species)	; A = species of third pkmn
	ld HL, TeamPKMN3
	ld C, 40
	rTextCoord(12, 32)
	call PartyName
	rCopyBuffSect(11, 32, 48, 8)
	
	ld A, (PartySel)
	call PartyOpen
	ei
	ret
	
PartyName:					; subroutine - disps name, A = pkmn species, HL = pt to data, BC = HP bar pos, text coords set
	push BC				; save BC (C needed for y pos)
	push HL				; save HL (pt to data)
	RCall( cGetName3 )			; HL = pt to name of pkmn in party
	ld B, 10				; max 10 chars
	RCall( cArcTextN )			; disp name, text coords already set
	pop HL					; get HL (pt to data)
	push HL				; save HL again
	RCall( cDispGenderCoord )		; disp gender at text coords (changed by ArcTextN)
	pop DE					; get DE (pt to data)
	pop HL					; HL = pos of HP bar
PartyNameHP:					; L = y pos of HP bar
	push HL				; save HL (y pos)
	push DE				; save HL (pt to data) as DE
	ld H, 25				; x pos always 25
	xor A					; A = 0 for DispHPX
	RCall( cDispHP )			; disp HP bar, pass HL as coords, DE as pt to data
	pop HL					; HL = pt to data
	
	ld BC, PKMN_Status			; GetPKMNStatus
	add HL, BC
	ld A, (HL)				; A = status ailment
	pop HL					; L = y pos
	dec L					; status icon is 1 pixel higher than HP bar
	ld H, 12				; x pos always 12
	
	RCall( cDispStatusX )		; disp status (A) at location (HL)
	ret
	
PartyClose:					; subroutine - input: A = selection, close pokeball, preserves AF
	cp 3
	jr nz, PartyCloseBall		; if opt = 3 (cancel opt)
	
	push AF
	rInvRectD(62, 56, 33, 7)		; clear cancel opt by invert buffer 2 + copy to buffer 1
	rCopyBuffSect2(62, 56, 33, 7)
	pop AF
	ret
	
PartyCloseBall:
	call PartyPokeballO
	jr PartyPokeballC
	
PartyOpen:					; subroutine - input: A = selection, open pokeball, preserves AF
	cp 3
	jr nz, PartyOpenBall			; if opt = 3 (cancel opt)
	
	push AF
	rDrawRectW(62, 56, 33, 7) 		; fill cancel opt by clear buffer 1 + invert buffer 2
	rInvRectD(62, 56, 33, 7)		; //TODO: optimize by putting parameter loads into routine?
	pop AF
	ret
	
PartyOpenBall:
	;//TODO: fill in box
	call PartyPokeballC
	;jr PartyPokeballO
	
PartyPokeballO:				; subroutine - input: A = selection, XOR draws open pokeball, preserves A
	push AF
	ld H, A
	ld L, 15
	call MultHL
	ld A, 3
	add A, L				; A = (A * 15) + 3
	ld L, A
	xor A
	ld IX, sPokeballO
	RCall( cGSLargeSprite )		; draws sPokeballO at (0, A)
	pop AF
	ret
	
PartyPokeballC:				; subroutine - input: A = selection, XOR draws closed pokeball, preserves A
	push AF
	ld H, A
	ld L, 15
	call MultHL
	ld A, 4
	add A, L				; A = (A * 15) + 4
	ld L, A
	xor A
	ld IX, sPokeballC
	RCall( cgsLargeSprite )		; draws sPokeballC at (0, A)
	pop AF
	ret
	
PartyGetPKMN:					; subroutine - gets pt to selected pkmn (PartySel), returns HL
	ld A, (PartySel)
	ld H, A
	ld L, BYTES_PER_TPKMN
	call MultHL
	ld DE, TeamPKMN1
	add HL, DE				; HL = pt to pkmn data, (TeamPKMN1 + A * 30)
	ret
	
PartyDisp0:					; subroutine - disps message in textbox, w/ name of first pkmn in party, input: HL, DE
	ld BC, PartySel
	ld A, (BC)
	push AF				; save (PartySel)
	xor A
	ld (BC), A				; set (PartySel) to pkmn currently in battle
	call PartyDisp			; disp message, saves BC
	pop AF
	ld (BC), A				; restore (PartySel)
	ret
	
PartyDisp3:					; subroutine - disps message in textbox, message = input as HL, waits for keypress
	call PartyDispInit
	jr PartyDisp21
	
PartyDisp:					; subroutine - disps selected pkmn name, message (input as HL / DE), then waits for keypress, saves BC
	di
	push BC
	push DE
	push HL
	call PartyDispInit			; clear text area, set text coordinates
	call PartyGetPKMN			; HL = pt to selected pkmn
	RCall( cGetName )			; HL = pt to pkmn name
	ld B, 10				; B = 10 (max pkmn name length)
	RCall( cArcTextN )
	pop HL
	RCall( cArcText )
	pop HL
	pop BC
PartyDisp2:					; disps HL msg at bottom of textbox
	rTextCoord(2, 56)			; BC not changed
PartyDisp21:
	push BC
	RCall( cArcText )
	rCopyBuffSect(1, 49, 58, 14)
	pop BC
	ei
	
PartyDispLp:					; wait until 2nd or Enter pressed
	push BC
	RCall( cWait )			; wait routine
	pop BC
	ret
	
PartyDispInit:				; subroutine for PartyDisp, saves HL
	push HL
	rDrawRectW3(1, 49, 58, 14)		; clear text area
	rTextCoord(2, 49)			; set coords
	pop HL
	ret
	