; ===============================================================
; Variable Storage (Safe RAM)
; ===============================================================

; ---------------------------------------------------------------
; Flags
; ---------------------------------------------------------------

; Asm_Flag1

#define LineColor		0				; 1 = black, 0 = white
								; default = 1 (set)
#define ROMFlag		1				; 1 = ROM, 0 = RAM
								; default = 0 (res)
#define BlinkerOn		2				; 1 = on; 0 = off
								; default = 0 (off)
#define TextROM		3				; 1 = ROM, 0 = RAM
								; default = 1 (ROM)
#define TempFlag		4				; temp flag
								; used by blinker
#define TurnFlag		5				; 1 = Player, 0 = Opp
#define PKMNFlag		6				; 1 = Player, 0 = Opp
								; used by textbox, dispstatus, dodamage
#define SwapFlag		7				; 1 = Swap, 0 = Rent
								; used by swapteam / selectteam
								
; Asm_Flag2

#define SwapPlayer		0				; 1 = player swap, 0 = accept opp pkmn
#define LineInvert		1				; 1 = invert, 0 = normal
#define Buffer2		2				; 1 = buffer 2 (Save), 0 = buffer 1 (Plot)
#define HealFlag		3				; 1 = heal, 0 = damage, used by dodamage
; bit 4 of Asm_Flag2 can't be used?
#define FrontFlag		5				; 1 = front sprites, 0 = back sprites
#define SilenceFlag		6				; 1 = silence textbox, 0 = display text (Textbox)
#define SwitchFlag		7				; 1 = switch after faint, 0 = switch in battle

; Asm_Flag3

#define CursorFlag		0				; 1 = cursor is displayed, 0 = blank
#define InfoFlag		1				; 1 = display info w/ status, HP, 0 = info

; ---------------------------------------------------------------
; AppBackupScreen (768 @ $9872)
; General and Temporary data
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Main Menu
; ---------------------------------------------------------------

#define MenuOpt		AppBackupScreen		; menu option #
								; 0 = Play
								; 1 = Record
								; 2 = Info
								; 3 = Quit

; ---------------------------------------------------------------
; Select Round
; ---------------------------------------------------------------

#define SelectRnum		AppBackupScreen + 3		; selected round # (1-5)

#define BattleNum		AppBackupScreen + 4		; Battle # (2 bytes, 1 relative)
								; Streak = BattleNum - 1
#define RoundNum		AppBackupScreen + 6		; Current Round # (1-5)
								; If Round > 5, # = 5
#define MatchNum		AppBackupScreen + 7		; Current Match # (1-7)

; ---------------------------------------------------------------
; Select Team
; ---------------------------------------------------------------

#define SelectTeamArr	AppBackupScreen + 8		; Array of 6 temp pkes (18 bytes)
								; pke.byte1 = tier (1-5)
								; pke.byte2 = #
								; pke.byte3 = personality id
								;    id.bit0 = gender
								;    id.bit1 = ability (0 first; 1 second)
#define SelectTeamNum	AppBackupScreen + 26		; cursor selected pkmn # (0 - SelectMax) ($988C)
#define SelectTeamOptNum	AppBackupScreen + 27		; selected opt # (0-2) or (0-1)
								; 0 = Summary
								; 1 = Rent
								; 2 = Back OR
								; 0 = Yes
								; 1 = No
#define SelectTeamQuan	AppBackupScreen + 28		; # of pokes selected (0-3), must be adjacent to selteam1
#define SelectTeam1		AppBackupScreen + 29		; first selection (0-5)
#define SelectTeam2		AppBackupScreen + 30		; second selection (0-5)
#define SelectTeam3		AppBackupScreen + 31		; third selection (0-5)
								; AppBackupScreen + 31/32 used for filler bytes
#define InfoPage		AppBackupScreen + 34		; Current page of info menu
								; 0 = Info
								; 1 = Skills
								; 2 = Moves
#define ConvStr		AppBackupScreen + 35		; location of converted str for IntStr (6 bytes max)
#define InfoMoveNum		AppBackupScreen + 41		; current move highlighted (0-3)

; ---------------------------------------------------------------
; Research
; ---------------------------------------------------------------

#define ResearchTypeDat	AppBackupScreen + 42		; array of pke types (17 bytes)
#define ResearchStyleDat	AppBackupScreen + 59		; array of move styles (7~ bytes) ($98AD)
#define PKMNDATArom		AppBackupScreen + 70		; ROM page of data program ($98B8)
#define PKMNDATAoff		AppBackupScreen + 71		; offset of data program (2 bytes)

#define ResearchOpt		AppBackupScreen + 100	; selected opt # (0 - 1 | 2 - 4)
								; 2 = Swap | 0 = Rent
								; 3 = Cont | 1 = Quit
								; 4 = Quit
#define SelectMax		AppBackupScreen + 101	; (max # pkmn) - 1 for rent / swap
								; 5 for rent
								; 2 for swap
#define PrevOppTeam		AppBackupScreen + 102	; team of previous opponent (9 bytes)
								; same pke format as TeamData
#define SwapPKMN		AppBackupScreen + 111	; selected pkmn to swap in player team (1 byte)

; ---------------------------------------------------------------
; Battle
; ---------------------------------------------------------------

#define BattleMenuX		AppBackupScreen + 73		; x pos for battle menu (1 byte)
#define BattleMenuY		AppBackupScreen + 74		; y pos for battle menu (1 byte)
#define BatAtkX		AppBackupScreen + 75		; x pos for attack menu (1 byte)
#define BatAtkY		AppBackupScreen + 76		; y pos for attack menu (1 byte)
#define CurMovePlayer	AppBackupScreen + 77		; stored player move for turn (1 byte)
#define CurMoveOpp		AppBackupScreen + 78		; stored opp move for turn (1 byte)
#define AtkMove		AppBackupScreen + 79		; attacker move ($FD - $FF for switch) (1 byte)
#define TargetMove		AppBackupScreen + 80		; target move (1 byte)
#define AtkPKMN		AppBackupScreen + 81		; attacker pkmn, pt to battle data (2 bytes)
#define TargetPKMN		AppBackupScreen + 83		; target pkmn, pt to battle data (2 bytes)

; ---------------------------------------------------------------
; Party
; ---------------------------------------------------------------

#define PartySel		AppBackupScreen + 112	; selected pkmn / option in party (0 - 2, 3 = Cancel) ($98E2)
#define PartyOptSel		AppBackupScreen + 113	; selected option in party menu (0 - 2)
								; 0 = Shift
								; 1 = Summary
								; 2 = Cancel
#define PartySwitch		AppBackupScreen + 114	; pkmn in party to switch to
								; 0   = none, no switch
								; $FF = first pkmn
								; $FE = second pkmn
								; $FD = third pkmn
#define PartyPrevSel		AppBackupScreen + 115	; selected pkmn previous to cancel opt (0 - 2, 3 = Cancel)

; ---------------------------------------------------------------
; Misc
; ---------------------------------------------------------------

#define DrawBuff		AppBackupScreen + 1		; Buffer to draw in (2 bytes)
#define TxtTemp		AppBackupScreen + 85		; RAM location of text (2 bytes)
#define LocTemp		AppBackupScreen + 87		; Temp storage of location, DoDamage (2 bytes)
#define DamPix		AppBackupScreen + 89		; Temp storage of damage pix, DoDamage (1 byte)
#define TextStat		AppBackupScreen + 90		; stat to display (0 = HP, 1 = atk...5 = spd)
#define TextNum		AppBackupScreen + 91		; num to display
#define NumText		AppBackupScreen + 92		; char code of num to display, NBTS (2 bytes)
#define TurnNum		AppBackupScreen + 95		; turn number (1 for first or 2 for second)
#define TextMove		AppBackupScreen + 96		; move ID to display 
#define StackPt		AppBackupScreen + 97		; pt to first empty entry in Stack (2 bytes) ($98D1)

#define TextItem		AppBackupScreen + 116	; item ID to display
#define Style			AppBackupScreen + 117	; opp style, set by ResearchStyle (0 - 8)
#define PKMNPICFrom		AppBackupScreen + 118	; ROM page of front pic program
#define PKMNPICFoff		AppBackupScreen + 119	; offset of front pic program (2 bytes)
#define PKMNPICBrom		AppBackupScreen + 121	; ROM page of back pic program
#define PKMNPICBoff		AppBackupScreen + 122	; offset of back pic program (2 bytes)
#define SaveVar		AppBackupScreen + 124	; 0 by default, 1 if using save data
#define RecordFlag		AppBackupScreen + 125	; 1 = new record, 0 = view results
#define BuffSize		AppBackupScreen + 126	; current size of input buffer
#define CursorCount		AppBackupScreen + 127	; number of times to stall before blinking cursor
#define BufferPt		AppBackupScreen + 128	; pointer to buffer for name input (2 bytes)
#define AlphaNum		AppBackupScreen + 130	; 0 = upper, 1 = lower, 2 = number
#define CursorChar		AppBackupScreen + 131	; char code of cursor to display
#define TextSize		AppBackupScreen + 132	; size of text to decode
#define TextAbil		AppBackupScreen + 133	; ability ID to display
#define SaveSP		AppBackupScreen + 134	; saved SP value, used by quit routine
#define SaveContrast		AppBackupScreen + 136	; saved contrast value

#define DebugStage		AppBackupScreen + 139	; debug stage value
#define LastPC		AppBackupScreen + 140	; last value of PC, recorded by interrupt (2 bytes) ($98FE)
#define DebugNum		AppBackupScreen + 140	; debug num (0 = none) ($98FE) ;;;
								; 1 = Battle, 2 = Turn, 3 = Attack, 4 = Attack2, 
								; 5 = CalcDamage, 6 = Effect, 7 = EOT, 8 = Textbox

;AppBackupScreen + 142 to 
;AppBackupScreen + 398 reserved for interrupt jump table

; ---------------------------------------------------------------
; Conversion data
; ---------------------------------------------------------------

#define Buffer		AppBackupScreen + 400	; Temporary buffer ($9A02). Used by ArcSprite, ArcText, ArcPKMN, RSwapRAM routine

#define block9		AppBackupScreen + 400	; block for FP conv routines (9 bytes)
#define block18		AppBackupScreen + 409	; block for FP conv routines (18 bytes)
#define block18A		AppBackupScreen + 427	; block for FP conv routines (18 bytes)
#define sig1			AppBackupScreen + 445	; FP conv data (1 byte)
#define LengthBCD		AppBackupScreen + 446	; FP conv data (1 byte)

;AppBackupScreen + 552 to /555/ 735 (740) reserved for interrupt code

;#define Stack			AppBackupScreen + 736	; Stack for RCall, each entry is 2-byte pt to routine, supports ~16 entries ($9B56)
#define Stack			AppBackupScreen + 745

; ---------------------------------------------------------------
; SaveSScreen (768)
; Grayscale buffer
; ---------------------------------------------------------------

; savesscreen reserved for greyscale buffer

; ---------------------------------------------------------------
; StatVars (531 @ $8A3A)
; Battle Mechanics data
; ---------------------------------------------------------------

#define TeamData		StatVars 			; Your team (9 bytes)
								; pke.byte1 = tier (1-5)
								; pke.byte2 = #
								; pke.byte3 = personality id
								;    id.bit0 = gender
								;    id.bit1 = ability (0 first; 1 second)
#define OppTeamData		StatVars + 9			; Opponent's team (9 bytes)
								; same pke format as TeamData
#define TeamPKMN1		StatVars + 18			; Status of 1st PKMN in player team (31 bytes) ($8A4C)
								; 00 = Tier #
								; 01 = Poke #
								; 02 = PID
								; 03 = Species ID
								; 04 = Move1 - Move4
								; 08 = Item
								; 09 = Nature
								; 10 = Max HP
								; 12 = Atk
								; 14 = Def
								; 16 = SAtk
								; 18 = SDef
								; 20 = Spd
								; 22 = Ability
								; 23 = Remaining HP
								; 25 = Status
								; 26 = PP1 - PP4
								; 30 = SLP count (0 by default, 1-7)
#define TeamPKMN2		StatVars + 49			; 2nd PKMN in player team, same format as 1st, must be contiguous
#define TeamPKMN3		StatVars + 80			; 3rd PKMN in player team, same format as 1st
#define OppPKMN1		StatVars + 111		; 1st PKMN in opponent team, same format as 1st ($8AA9)
#define OppPKMN2		StatVars + 142		; 2nd PKMN in opponent team, same format as 1st
#define OppPKMN3		StatVars + 173		; 3rd PKMN in opponent team, same format as 1st
#define PlayerPKMN		StatVars + 204		; Detailed status of current player pkmn (80 bytes) ($8B06)
								; 00 = Info (^)
								; 31 = Team # (1-3 | 4-6)
								; 32 = Atk boost (-6 - 0 - 6)
								; 33 = Def boost (-6 - 0 - 6)
								; 34 = SAtk boost (-6 - 0 - 6)
								; 35 = SDef boost (-6 - 0 - 6)
								; 36 = Spd boost (-6 - 0 - 6)
								; 37 = Acc boost (-6 - 0 - 6)
								; 38 = Eva boost (-6 - 0 - 6)
								; 39 = Bad PSN counter (0 by default, 1 => 1/16 dmg)
								; 40 = Yawn (0 by default, or 1-2 turns)
								; 41 = Last move ID ($FF if none)
								; 42 = Disabled move ID ($FF if none)
								; 43 = Disabled move counter (0 - 7)
								; 44 = Leech seed status (0 if no, 1 if yes)
								; 45 = Encore move ID ($FF if none)
								; 46 = Encore move counter (0 - 6)
								; 47 = Infatuation (0 if no, 1 if yes)
								; 48 = Confusion (0 if no, 1 if last turn, else 1-4 turns)
								; 49 = Prev escape, affects foe (0 if no, 1 if yes)
								; 50 = Perish counter (0 if no, else 1-4 => 0-3 turns)
								; 51 = Protect status (0 if no, 1 if yes)
								; 52 = Counter (protect, detect, endure) (n = num turns)
								; 53 = Endure status (0 if no, 1 if yes)
								; 54 = Stall status (0 = none, 1 = bounce, 2 = dig, 3 = dive, 4 = fly)
								; 55 = Flinch (0 if no, 1 if yes)
								; 56 = Focus (0 if no, 1 if yes)
								; 57 = HP25 (0 if no, 1 if HP <= 25% max)
								; 58 = HP50 (0 if no, 1 if HP <= 50% max)
								; 59 = Charge (0 if no, 1 if yes)
								; 60 = Trap (0 if no, 1 if last turn, else 1-5 turns)
								; 61 = Solar (0 if no, 1 if solarbeam is charged)
								; 62 = Rollout (0 if no, else number of consecutive turns)
								; 63 = Defense Curl (0 if no, 1 if yes)
								; 64 = Ingrain (0 if no, 1 if yes)
								; 65 = Torment (0 if no, 1 if yes)
								; 66 = Torment move ($FF if no, move ID if yes)
								; 67 = Truant (0 if no, 1 if loaf on next turn)
								; 68 = Trace (if abil = trace, then traced abil, else 0)
								; 69 = Flash Fire (0 if no, 1 if activated)
								; 70 = Choice (0 for default, 1 if no choice)
								; 71 = Skill swap (0 if no, ability ID | 0x80 if swapped)
								; 72 = Destiny bond (0 if no, 1 if current turn, 2 if next turn)
								; 73 = Grudge (0 if no, 1 if current turn, 2 if next turn)
								; 74 = Nightmare (0 if no, 1 if yes)
								; 75 = Sky Attack (0 if no, 1 if sky attack is charged)
								; 76 = Duration (0 if no, else number of turns left)
								; 77 = Substitute HP (0 if no, else remaining HP of sub)
								
#define OppPKMN		StatVars + 284		; Status of current opponent pkmn (80 bytes) ($8B56)
								; Same format as PlayerPKMN
								
#define Field			StatVars + 364		; Status of field (?? bytes) ($8BCE)
#define Weather		StatVars + 364		; Status of weather
								; 0 = None
								; 1 = Sun
								; 2 = Rain
								; 3 = Sandstorm
								; 4 = Hail
#define WeatherCount		StatVars + 365		; Duration of weather in turns (0 = none, $FF = indefinite)
#define UserField		StatVars + 366
#define UserLight		StatVars + 366		; User team light screen status (0 = no, else = turn count + 1)
#define UserReflect		StatVars + 367		; User team reflect status (0 = no, else = turn count + 1)
#define UserSafeguard	StatVars + 368		; User safeguard status (0 = no, else = turn count + 1)
#define UserSpikes		StatVars + 369		; User's spikes layer on opp's side (0 = no, else # spikes layers (1-3))
#define UserFutureTurn	StatVars + 370		; Turns until user is hit w/ future sight (0 = no, else # turns)
#define UserFutureSpAtk	StatVars + 371		; Sp Atk of future sight attack
#define UserFutureSpDef	StatVars + 373		; Sp Def of future sight attack
#define UserFutureHit	StatVars + 375		; Whether future sight hits or misses (0 = miss, 1 = hit)
#define UserWish		StatVars + 376		; Wish status (0 = none, 1 | 2 = turns until wish)
#define OppField		StatVars + 380
#define OppLight		StatVars + 380		; Opp team light screen status (0 = no, 1 = yes)
#define OppReflect		StatVars + 381		; Opp team reflect status (0 = no, 1 = yes)
#define OppSafeguard		StatVars + 382		; Opp safeguard status (0 = no, 1 = yes)
#define OppSpikes		StatVars + 383		; Opp's spikes layer on user's side (0 = no, else # spikes layers)
#define OppFutureTurn	StatVars + 384		; Turns until opp is hit w/ future sight (0 = no, else # turns)
#define OppFutureSpAtk	StatVars + 385		; Sp Atk of future sight attack
#define OppFutureSpDef	StatVars + 387		; Sp Def of future sight attack
#define OppFutureHit		StatVars + 389		; Whether future sight hits or misses (0 = miss, 1 = hit)
#define OppWish		StatVars + 390		; Wish status (0 = none, 1 | 2 = turns until wish)

#define CritBool		StatVars + 408		; critical hit boolean (0 = false / no crit, 1 = true / crit)
#define TypeEff		StatVars + 409		; type effectiveness (0 | 1 | 2 | 4 | 8 | 16)
#define PlayerFainted	StatVars + 410		; player pkmn fainted boolean
#define OppFainted		StatVars + 411		; opp pkmn fainted boolean 
#define NextPlayerPKMN	StatVars + 412		; next selected player pkmn
#define NextOppPKMN		StatVars + 413		; next selected opp pkmn
#define BattleWin		StatVars + 414		; winner of battle
								; (0 by default, 1 if player wins, $FF if opp wins, $FE if forfeit)
#define EOTPriority		StatVars + 415		; 0 = opp priority, 1 = player priority
#define HitRepeat		StatVars + 416		; num of repeated hits (0 = none, else 1+ repeats)
#define Damage		StatVars + 417		; total damage calculated by CalcDamage (2 bytes)
#define HitMiss		StatVars + 419		; hit / miss calculated by CalcMiss (0 = miss, 1 = hit)
#define FocusBandAct		StatVars + 420		; whether focus band is activated (0 = no, 1 = yes)
#define LastDmg		StatVars + 421		; total damage done in last move (2 btes)
#define PKMNSwitch		StatVars + 423		; 0 = no switch, 1 = switch by roar
#define PKMNFNT		StatVars + 424		; 0 = no FNT, 1 = FNT during routine
#define EndureAct		StatVars + 425		; whether endure is activated (0 if no, 1 if yes)
#define LevitateAct		StatVars + 426		; whether levitate is activated (0 if no, 1 if yes)
#define DamageEff		StatVars + 427		; damage effect, calculated by CalcDamage (0 = dam, 1 = no pow, 2 = no eff)
#define TrgtProtect		StatVars + 428		; whether target used protect calculated by IsTargetProtect (0 = no, 1 = yes)

#define PKMNSaveData		StatVars + 430		; data from pkmn save appvar (84 bytes) ($8BE8)
								; 0 = Name (8 bytes)
#define PKMNSaveWins		PKMNSaveData + 8		; 8 = Wins (2 bytes)
								; 10 = Swap (2 bytes)
								; ...
#define PKMNSaveTeam		PKMNSaveData + 60		; 60 = Team data (9 bytes)
#define PKMNSaveOpp		PKMNSaveData + 69		; 69 = Opp data (9 bytes)
#define PKMNSavePrev		PKMNSaveData + 78		; 78 = Prev data (9 bytes)
#define PKMNSaveBattle	PKMNSaveData + 87		; 87 = Battle number (2 bytes)
#define PKMNSaveRound	PKMNSaveData + 89		; 89 = Round number (1 byte)
#define PKMNSaveMatch	PKMNSaveData + 90		; 90 = Match number (1 byte)
#define PKMNSaveSwap		PKMNSaveData + 91		; 91 = Swaps (2 byte)
#define PKMNSaveCurrent	PKMNSaveData + 93		; 93 = Current (1 if in battle, 0 if forfeit / lose)
#define PKMNSaveBP		PKMNSaveData + 94		; 94 = Battle Points (2 byte)
								
; ---------------------------------------------------------------
; TextShadow (128) may be used as last resort
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; TempSwapArea (323) may be used when not loading archived save
; writing to TempSwapArea causes MirageOS to quit to LINK error
; ---------------------------------------------------------------
